import unittest
from datetime import timedelta

class TestCoerceTimedelta(unittest.TestCase):
    def _callFUT(self, value):
        from anykeystore.utils import coerce_timedelta
        return coerce_timedelta(value)

    def test_coerce_with_int(self):
        result = self._callFUT(20)
        self.assertEqual(result, timedelta(seconds=20))

    def test_coerce_with_timedelta(self):
        dt = timedelta(seconds=1)
        result = self._callFUT(dt)
        self.assertEqual(result, dt)



import sys

# True if we are running on Python 3.
PY3 = sys.version_info[0] == 3

try:
    import cPickle as pickle
except ImportError: # pragma: no cover
    import pickle

if PY3: # pragma: no cover
    def iteritems_(d):
        return d.items()
    def itervalues_(d):
        return d.values()
    def iterkeys_(d):
        return d.keys()
else:
    def iteritems_(d):
        return d.iteritems()
    def itervalues_(d):
        return d.itervalues()
    def iterkeys_(d):
        return d.iterkeys()



import unittest

class TestStore(unittest.TestCase):

    def test_create_store(self):
        from anykeystore import create_store
        store = create_store('memory')

        from anykeystore.backends.memory import MemoryStore
        self.assertTrue(isinstance(store, MemoryStore))

    def test_create_store_from_settings(self):
        from anykeystore import create_store_from_settings
        store = create_store_from_settings({'store': 'memory'})

        from anykeystore.backends.memory import MemoryStore
        self.assertTrue(isinstance(store, MemoryStore))

    def test_create_store_from_settings_with_prefix(self):
        from anykeystore import create_store_from_settings
        store = create_store_from_settings(
            {'any.store': 'memory'}, prefix='any.')

        from anykeystore.backends.memory import MemoryStore
        self.assertTrue(isinstance(store, MemoryStore))



import sys

from anykeystore.exceptions import ConfigurationError

def _load_backend(name):
    try:
        module_name = 'anykeystore.backends.%s' % name
        __import__(module_name)
        module = sys.modules[module_name]
        if hasattr(module, 'backend'):
            backend = module.backend
        else:
            module = _load_entry_point(name)
            if module is None:
                raise ConfigurationError(
                    'Could not determine backend for "%s"' % name)
        return backend
    except ImportError:
        module = _load_entry_point(name)
        if module is None:
            return module
        raise ConfigurationError(
            'Could not determine backend for "%s"' % name)

def _load_entry_point(name):
    try:
        import pkg_resources
    except ImportError:
        return None

    for res in pkg_resources.iter_entry_points('anykeystore.backends'):
        if res.name == name:
            return res.load()

def create_store(name, **kwargs):
    backend_cls = _load_backend(name)
    return backend_cls(**kwargs)

def create_store_from_settings(settings, prefix='', **kwargs):
    plen = len(prefix)
    for k, v in settings.items():
        if k.startswith(prefix):
            kwargs[k[plen:]] = v
    name = kwargs.pop('store')
    return create_store(name, **kwargs)



import unittest

class TestRedisStore(unittest.TestCase):

    def _makeOne(self, url='sqlite://', **kw):
        from anykeystore.backends.sqla import SQLStore
        store = SQLStore(url, **kw)
        store.create()
        return store

    def test_it(self):
        store = self._makeOne()
        store.store('foo', 'bar')
        value = store.retrieve('foo')
        self.assertEqual(value, 'bar')

    def test_it_delete(self):
        store = self._makeOne()
        store.store('foo', 'bar')
        store.delete('foo')
        self.assertRaises(KeyError, store.retrieve, 'foo')

    def test_it_old(self):
        store = self._makeOne()
        store.store('foo', 'bar', expires=-1)
        self.assertRaises(KeyError, store.retrieve, 'foo')

    def test_it_purge(self):
        store = self._makeOne()
        store.store('foo', 'bar', expires=-1)
        store.purge_expired()
        self.assertRaises(KeyError, store.retrieve, 'foo')



from datetime import datetime

from sqlalchemy import engine_from_config
from sqlalchemy import Column, MetaData
from sqlalchemy import String, PickleType, DateTime, Table
from sqlalchemy.sql import select, delete

from anykeystore.interfaces import KeyValueStore
from anykeystore.utils import coerce_timedelta


class SQLStore(KeyValueStore):
    """ Simple storage via SQLAlchemy.

    The store will automatically create a table object if one is not
    supplied. The automatically generated table is shown below. If a
    table is supplied directly, it must support the required columns
    `key`, `value`, and `expires`.

    .. code-block:: python

        table = Table(table_name, metadata,
            Column('key', String(200), primary_key=True, nullable=False),
            Column('value', Text(), nullable=False),
            Column('expires', DateTime()),
        )

    :param engine: A SQLAlchemy engine.
    :param table: Optional. The SQLAlchemy Table instance to be used for
                  storage. If this isn't supplied, a table is generated
                  automatically.
    :type table: sqlalchemy.Table
    :param table_name: Optional. The name of the table.
    :param metadata: Optional. The SQLAlchemy MetaData instance to hook the
                 generated table into.
    :type metadata: sqlalchemy.MetaData
    """
    def __init__(self, url, **kw):
        self.url = url
        self.table = kw.pop('table', None)
        if not self.table:
            table_name = kw.pop('table_name', 'key_storage')
            meta = kw.pop('metadata', MetaData())
            self.table = Table(table_name, meta,
                Column('key', String(256), primary_key=True, nullable=False),
                Column('value', PickleType(), nullable=False),
                Column('expires', DateTime()),
            )
        kw['url'] = url
        self.engine = engine_from_config(kw, prefix='')

    def create(self):
        self.table.create(checkfirst=True, bind=self.engine)

    def retrieve(self, key):
        c = self.engine.connect()
        try:
            data = c.execute(select(
                [self.table.c.value, self.table.c.expires],
                self.table.c.key == key)).fetchone()
            if data:
                value, expires = data
                if expires is None or datetime.utcnow() < expires:
                    return value
        finally:
            c.close()
        raise KeyError

    def store(self, key, value, expires=None):
        expiration = None
        if expires:
            expiration = datetime.utcnow() + coerce_timedelta(expires)
        c = self.engine.connect()
        try:
            c.execute(
                self.table.insert(), key=key, value=value, expires=expiration)
        finally:
            c.close()

    def delete(self, key):
        c = self.engine.connect()
        try:
            c.execute(delete(self.table, self.table.c.key == key))
        finally:
            c.close()

    def purge_expired(self):
        c = self.engine.connect()
        try:
            c.execute(
                delete(self.table, self.table.c.expires < datetime.utcnow()))
        finally:
            c.close()

backend = SQLStore
